/*
 * Copyright 2006, XpertNet SARL, and individual contributors as indicated
 * by the contributors.txt.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 * @author ludovic
 * @author sdumitriu
 */
package com.xpn.xwiki.plugin.charts.tests;


import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.xpn.xwiki.XWiki;
import com.xpn.xwiki.XWikiConfig;
import com.xpn.xwiki.XWikiContext;
import com.xpn.xwiki.doc.XWikiDocument;
import com.xpn.xwiki.objects.BaseObject;
import com.xpn.xwiki.objects.BaseProperty;
import com.xpn.xwiki.objects.classes.BaseClass;

public class ObjectsTest extends TestCase {
	public ObjectsTest(String arg0) {
		super(arg0);
	}
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(ObjectsTest.class);
	}

	protected void setUp() throws Exception {
        this.config = new XWikiConfig();
        this.config.put("xwiki.store.class", "com.xpn.xwiki.plugin.charts.mocks.MockStore");
        this.config.put("xwiki.store.attachment.class", "com.xpn.xwiki.plugin.charts.mocks.MockStore");

        this.context = new XWikiContext();
        this.xwiki = new XWiki(this.config, this.context);
        this.context.setWiki(this.xwiki);
        
		this.xclass = TestHelper.createTableDataSourceClass(context);
		
		this.doc = TestHelper.createDocument("Main.Doc", "{table}\ncontent\n{table}", context);
	}

	protected void tearDown() throws Exception {
	}
	
	public void testObjectDocument() throws Exception {
		BaseObject xobject = TestHelper.defineTable(xclass, this.doc, this.context, 0, "A1-A1", false, false);
		Assert.assertEquals(this.doc.getFullName(), xobject.getName());
	}
	
	public void testListDocuments() throws Exception {
		List list = xwiki.getStore().searchDocumentsNames("", context);
		Assert.assertTrue(list.size()>=1);
		Assert.assertTrue(list.contains(doc));
		
		XWikiDocument doc1 = TestHelper.createDocument("XWikiTest.Doc1", "", context);
		XWikiDocument doc2 = TestHelper.createDocument("XWikiTest.Doc2", "", context);
		XWikiDocument doc3 = TestHelper.createDocument("XWikiTest.Doc3", "", context);

		list = xwiki.getStore().searchDocumentsNames("doc.fullName like 'XWikiTest.%'", context);
		Assert.assertEquals(3, list.size());
		Assert.assertTrue(list.contains(doc1));
		Assert.assertTrue(list.contains(doc2));
		Assert.assertTrue(list.contains(doc3));
	}

	public void testId() throws Exception {
		BaseObject obj = TestHelper.defineTable(xclass, this.doc, this.context, 0, "A1-A1", false, false);

		List list = xwiki.getStore().search("from "+BaseObject.class.getName()
				+ " as obj where obj.id='"+obj.getId()+"'", 0, 0, context);
		Assert.assertEquals(1, list.size());

		BaseObject obj2 = (BaseObject)list.get(0);

		List propertyList = xwiki.getStore().search("from "+BaseProperty.class.getName()
				+ " as p where p.id.id='"+obj.getId()+"'", 0, 0, context);

		Iterator it = propertyList.iterator();
		while (it.hasNext()) {
			BaseProperty prop = (BaseProperty)it.next();
			obj2.addField(prop.getName(), prop);
		}
		
		Assert.assertEquals(obj, obj2);
	}
    
    private XWiki xwiki;
    private XWikiConfig config;
    private XWikiContext context;
    private BaseClass xclass;
    private XWikiDocument doc;
}
